/*
 *  Id: MHz.c,v 1.4 2001/05/21 18:58:01 davej Exp
 *  This file is part of x86info.
 *  (C) 2001 Dave Jones.
 *
 *  Licensed under the terms of the GNU GPL License version 2.
 *
 * Estimate CPU MHz routine by Andrea Arcangeli <andrea@suse.de>
 * Small changes by David Sterba <sterd9am@ss1000.ms.mff.cuni.cz>
 *
 * Modified for xcpuload by YOKOTA Hiroshi <yokota@netlab.is.tsukuba.ac.jp>
 */

/* $Id: cpu-ix86.c,v 1.3 2003/10/20 06:23:58 elca Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>

#include <inttypes.h>
/* typedef unsigned long long int   uint64_t; */

#include "cpu-load.h"

#define MAX(a,b) ((a) > (b) ? (a) : (b))

static inline uint64_t rdtsc(void)
{
	uint64_t x;

	asm volatile ("rdtsc" : "=A" (x));

	return x;
}

unsigned int estimate_MHz(void)
{
	struct timeval tvstart, tvstop;
	uint64_t       cycles[2];    /* gotta be 64 bit */
	unsigned int   microseconds; /* total time taken */
	int	       freq;

	gettimeofday(&tvstart, NULL); /* get this function in cached memory */
	cycles[0] = rdtsc();
	gettimeofday(&tvstart, NULL);

	/* we don't trust that this is any specific length of time */
	usleep(1000);

	cycles[1] = rdtsc();
	gettimeofday(&tvstop, NULL);

	microseconds = ((tvstop.tv_sec - tvstart.tv_sec) * 1000000) +
		(tvstop.tv_usec - tvstart.tv_usec);

	freq = (cycles[1] - cycles[0]) / microseconds;
	if (freq < 0) {
		fprintf(stderr, "clock skew detected\n");
	}

	return MAX(freq, 0);
}

/* end */
