/* $XConsortium: xload.c,v 1.37 94/04/17 20:43:44 converse Exp $ */
/* $XFree86: xc/programs/xload/xload.c,v 1.2 2000/05/11 18:14:45 tsi Exp $ */
/* $Id: xcpuload.c,v 1.5 2003/11/12 16:06:45 elca Exp $ */
/*

Copyright (c) 1989  X Consortium

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall
not be used in advertising or otherwise to promote the sale, use or
other dealings in this Software without prior written authorization
from the X Consortium.

*/

/*
 * xcpuload - display CPU load average in a window
 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/StripChart.h>
#include <X11/Xmu/SysUtil.h>

#include <X11/xpm.h>
#include "xcpuload.xpm"

#include "cpu-load.h"

static char *ProgramName;

static void GetLoadPoint(Widget w, XtPointer closure, XtPointer call_data);
static void quit(Widget w, XEvent *event, String *params, Cardinal *num_params);

/*
 * Definition of the Application resources structure.
 */

typedef struct _XCpuLoadResources {
	Boolean show_label;
	Boolean show_mhz;
	Boolean show_graph;
} XCpuLoadResources;

/*
 * Command line options table.  Only resources are entered here...there is a
 * pass over the remaining options after XtParseCommand is let loose. 
 */

static XrmOptionDescRec options[] = {
 {"-scale",		"*load.minScale",	XrmoptionSepArg,	NULL},
 {"-update",		"*load.update",		XrmoptionSepArg,	NULL},
 {"-hl",		"*load.highlight",	XrmoptionSepArg,	NULL},
 {"-highlight",		"*load.highlight",	XrmoptionSepArg,	NULL},
 {"-labelstr",		"*label.label",		XrmoptionSepArg,	NULL},
 {"-jumpscroll",	"*load.jumpScroll",	XrmoptionSepArg,	NULL},
 {"-label",		"*showLabel",	        XrmoptionNoArg,       "True"},
 {"-nolabel",		"*showLabel",	        XrmoptionNoArg,      "False"},
 {"-mhz",		"*showMHz",		XrmoptionNoArg,	      "True"},
 {"-nomhz",		"*showMHz",		XrmoptionNoArg,	     "False"},
 {"-graph",		"*showGraph",		XrmoptionNoArg,	      "True"},
 {"-nograph",		"*showGraph",		XrmoptionNoArg,	     "False"},
};

/*
 * The structure containing the resource information for the
 * Xload application resources.
 */

#define Offset(field) (XtOffsetOf(XCpuLoadResources, field))

static XtResource my_resources[] = {
  {"showLabel", XtCBoolean, XtRBoolean, sizeof(Boolean),
     Offset(show_label), XtRImmediate, (XtPointer) TRUE},
  {"showMHz", XtCBoolean, XtRBoolean, sizeof(Boolean),
    Offset(show_mhz), XtRImmediate, (XtPointer) TRUE},
  {"showGraph", XtCBoolean, XtRBoolean, sizeof(Boolean),
    Offset(show_graph), XtRImmediate, (XtPointer) TRUE},
};

#undef Offset

static XCpuLoadResources resources;

static XtActionsRec xcpuload_actions[] = {
    { "quit",	quit },
};
static Atom wm_delete_window;

static Widget mhz_label;

/*
 * Exit with message describing command line format.
 */

void usage(void)
{
    fprintf (stderr, "usage:  %s [-options ...]\n\n", ProgramName);
    fprintf (stderr, "where options include:\n");
    fprintf (stderr,
      "    -display dpy            X server on which to display\n");
    fprintf (stderr,
      "    -geometry geom          size and location of window\n");
    fprintf (stderr, 
      "    -fn font                font to use in label\n");
    fprintf (stderr, 
      "    -scale number           minimum number of scale lines\n");
    fprintf (stderr, 
      "    -update seconds         interval between updates\n");
    fprintf (stderr,
      "    -labelstr string        annotation text\n");
    fprintf (stderr, 
      "    -bg color               background color\n");
    fprintf (stderr, 
      "    -fg color               graph color\n");
    fprintf (stderr, 
      "    -hl color               scale and text color\n");
    fprintf (stderr, 
      "    -jumpscroll value       number of pixels to scroll graph on overflow\n");
    fprintf (stderr, 
      "    -label / -nolabel       adds/removes the label from above the chart.\n");
    fprintf (stderr, 
      "    -mhz   / -nomhz         adds/removes the real clock speed label in MHz\n");
    fprintf (stderr, 
      "    -graph / -nograph       adds/removes the real clock speed graph\n");
    fprintf (stderr, "\n");
    exit(1);
}


int
main(int argc, char **argv)
{
	XtAppContext app_con;
	Widget toplevel, load, pane, label_wid;
	Pixmap icon_pixmap = None;
	char *label, host[256];
	static String fallback_resources[] = {
#include "XCpuLoad_ad.h"
	    NULL,
	};


	ProgramName = argv[0];

	XtSetLanguageProc(NULL, NULL, NULL);

	toplevel = XtVaOpenApplication(&app_con, "XCpuLoad",
				       options, XtNumber(options),
				       &argc, argv,
				       fallback_resources,
				       applicationShellWidgetClass,
				       NULL);
	if (argc != 1) { usage(); }

	XtVaGetApplicationResources(toplevel, (XtPointer) &resources, 
				    my_resources, XtNumber(my_resources),
				    NULL);
    	/*
     	 * This is a hack so that f.delete will do something useful in this
     	 * single-window application.
     	 */
    	XtAppAddActions (app_con, xcpuload_actions, XtNumber(xcpuload_actions));

	XtVaGetValues(toplevel, XtNiconPixmap, &icon_pixmap, NULL);
	if (icon_pixmap == None) {
		Pixmap my_icon, my_icon_mask;

		XpmCreatePixmapFromData(XtDisplay(toplevel),
					XtScreen(toplevel)->root,
					xcpuload_xpm,
					&my_icon, &my_icon_mask, NULL);

		XtVaSetValues(toplevel,
			      XtNiconPixmap, my_icon,
			      XtNiconMask,   my_icon_mask,
			      NULL);
    	}

	pane = XtVaCreateManagedWidget("paned", panedWidgetClass,
				       toplevel, NULL);

    	if (resources.show_label) {
		label_wid = XtVaCreateManagedWidget("label", labelWidgetClass,
						    pane, NULL);
  
		XtVaGetValues(label_wid, XtNlabel, &label, NULL);

		if ( strcmp("label", label) == 0 ) {
			(void) XmuGetHostname (host, 255);
			XtVaSetValues (label_wid, XtNlabel, host, NULL);
		}
	}

	load = XtVaCreateManagedWidget("load", stripChartWidgetClass,
				       pane, NULL);
	if (! resources.show_graph) {
		XtUnmanageChild(load);
	}

	XtAddCallback(load, XtNgetValue, GetLoadPoint, NULL);

	if (resources.show_mhz) {
		mhz_label = XtVaCreateManagedWidget("mhz", labelWidgetClass,
						    pane, NULL);
	}

    	XtRealizeWidget (toplevel);
    	wm_delete_window = XInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW",
				       False);
    	(void) XSetWMProtocols (XtDisplay(toplevel), XtWindow(toplevel),
			    	&wm_delete_window, 1);

	XtAppMainLoop(app_con);

	return 0;
}

static void
quit (Widget    w,
      XEvent   *event,
      String   *params,
      Cardinal *num_params)
{
    if (event->type == ClientMessage &&
        event->xclient.data.l[0] != wm_delete_window) {
        XBell (XtDisplay(w), 0);
        return;
    }

    XtDestroyApplicationContext(XtWidgetToApplicationContext(w));
    exit (0);
}

static void
GetLoadPoint(Widget    w,         /* unused */
	     XtPointer closure,   /* unused */
	     XtPointer call_data) /* pointer to (double) return value */
{
	double *mhz = (double *)call_data;
	char str[20];
	unsigned int tmp;

	tmp = estimate_MHz();

	*mhz = tmp / 100.0;

	if (resources.show_mhz) {
		snprintf(str, sizeof(str), "%d MHz", tmp);
		XtVaSetValues(mhz_label, XtNlabel, str, NULL);
	}

	return;
}

/* end */
